package Sim;

import java.util.Random;

/**
 * RandomLocationManager implements a LocationManager that 
 * will allow a node to move from a position to another
 * random position and pause.  After the pause it will continue
 * to move to another random potion.
 * 
 * @author Benjamin Willis
 */
public class RandomLocationManager implements LocationManager{
	private Location end;
	private Location start;
	private Double distance;
	private double startTime;
	private double endTime;
	private Random rand;
	private Double max;
	private Double pause = 5.0;
	private Double speed;

	/**
	 * Constructs a location manager that will return
	 * paths to random locations.
	 *
	 * @param start a position to start
	 * @param speed the speed which the node is moving
	 * @area the area of the world
	 * @seed random seed number
	 */
	public RandomLocationManager(Double speed, Integer area, Location start, Integer seed) {
		this.max = (Double) Math.ceil(Math.sqrt(area));
		this.rand = new Random(seed);
		this.end = new Location(rand.nextInt(max.intValue()),rand.nextInt(max.intValue()));
		this.start = start;
		this.startTime = 0;
		this.distance = start.getDistance(end);
		this.speed = speed;
		this.endTime = this.distance / speed + startTime; 
	}

	/**
	 * Returns the location produced by this manager at the given time t. The value of t passed in must
	 * be equal to or greater than the value of all previous t values passed into this method.
	 *
	 * @param t time to get the location at
	 * @return non-null generated Location
	 */
	public Location getLocation( double t ) {
		if(t <= this.endTime + pause && t >= this.endTime)
			{
			return this.end;
			}
		else
			{
			while(t > this.endTime)
				{
				newPath();
				}

			double weight = Math.abs((t - this.startTime) / ( this.endTime - this.startTime));
			return start.mult( 1 - weight ).add( end.mult( weight ) );
			}
	}
	
	/**
	 * Construct a new start, end locations as well as 
	 * the other parameters required to change course.
	 */
	private void newPath(){
		this.start = this.end;
		this.end = new Location(rand.nextInt(max.intValue()),rand.nextInt(max.intValue()));
		this.distance = start.getDistance(end);
		this.startTime = this.endTime + pause;
		this.endTime = this.distance / speed + startTime;  
	}
}
