/*
 * RouteRequestCache.java
 *
 * Created on May 7, 2007, 4:33 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package Sim.Routing.Champ;
import Sim.Routing.DSR.Route;
import java.util.*;

/**
 *
 * @author Travis
 */
public class RouteRequestCache {
    
    private LinkedList <RouteRequested> routes = new LinkedList
            <RouteRequested>();
    int numPaths;
    
    
    /** Creates a new instance of RouteRequestCache 
     @param numPaths*/
    public RouteRequestCache(int numPaths) {
        this.numPaths = numPaths;
    }
    
    
    /**
     *setRouteStatus will set the status of the route to status
     *@source
     *@destination
     *@sequenceNumber
     *@status
     */
    public void setRouteStatus(int source, int destination,int sequenceNumber,
            boolean status){
        this.getRoute(source,destination,sequenceNumber).setRequestStatus(status);
    }
    
    
    
    /**
     *replacePreviousHops will replace the old list of previous hops 
     *and replace with the previousHop passed to it
     *@param source
     *@param destination
     *@param sequenceNumber
     *@param previousHop
     */
    public void replacePreviousHops(int source,int destination, int sequenceNumber,
            int previousHop){
        RouteRequested route=this.getRoute(source,destination,sequenceNumber);
        LinkedList <Integer>previousHops = new LinkedList<Integer>();
        previousHops.add(new Integer(previousHop));
        route.setPreviousHops(previousHops);
    }
    
    
    
    
    /**
     *getRouteMinForwardCount
     *@param source original source
     *@param destination from source to destination
     *@param sequenceNumber of RREQ
     *@return numberHops source to destination of -1 if error
     */
    public int getRouteMinForwardCount(int source,int destination,int sequenceNumber){
        int numberHops = -1;
        for(int i = 0; i < routes.size(); i++){
            if(routes.get(i).getDestination() == destination &&
                    routes.get(i).getSource() == source &&
                    routes.get(i).getSequenceNumber() == sequenceNumber){
                numberHops = routes.get(i).getMinForwardCount();
            }
        }
        return numberHops;
    }
    
    
    /**
     *updateForwardCount
     *@param souce
     *@param destination
     *@param sequenceNumber
     *@param minForwardCount to update too
     */
    public void updateForwardCount(int source,
            int destination,
            int sequenceNumber,
            int minForwardCount){
        for(int i = 0; i < routes.size(); i++){
            if(routes.get(i).getDestination() == destination &&
                    routes.get(i).getSource() == source &&
                    routes.get(i).getSequenceNumber() == sequenceNumber){
                routes.get(i).setMinForwardCount(minForwardCount);    //set minforward for route identified
            }
        }
    }
    
    /**
     *getAgeOfRoute
     *@param source
     *@param destination
     *@param sequenceNumber
     *@return age of route
     */
   // public int getAgeOfRoute(int source, int destination,int sequenceNumber){
    //    for(int i = 0; i < routes.size(); i++){
    //        if(routes.get(i).getDestination() == destination &&
   //                 routes.get(i).getSource() == source &&
   //                routes.get(i).getSequenceNumber() == sequenceNumber){
   //             routes.get(i).getAge();
  //          }
        
  //      }
  //  }
    
    
    /**
     *addPreviousHopToRoute()
     *@param source
     *@param destination
     *@param sequenceNumber
     *@param previousHop
     */
    public void addPreviousHopToRoute(int source,int destination,int sequenceNumber,
            int previousHop){
        for(int i = 0; i < routes.size(); i++){
            if(routes.get(i).getDestination() == destination &&
                    routes.get(i).getSource() == source &&
                    routes.get(i).getSequenceNumber() == sequenceNumber){
                
                LinkedList <Integer> previous = routes.get(i).getPreviousHops();
                if(!previous.contains(previousHop)){
                    previous.add(previousHop);
                    routes.get(i).setPreviousHops(previous);
                }
            }
        }
    }
    
    
    /**getRoute identified by triple
     *@param source
     *@param destination
     *@param sequenceNumber
     *@return requestedRoute of null if not present
     */
    public RouteRequested getRoute(int source, int destination,int
            sequenceNumber){
	            
        RouteRequested requestedRoute = null;
        if(routes != null){
            for(int i = 0; i < routes.size(); i++){
                if(routes.get(i).getSource() == source &&
                        routes.get(i).getDestination() == destination &&
                        routes.get(i).getSequenceNumber() == sequenceNumber){
                    requestedRoute = routes.get(i);
                    i = routes.size();  // exit loop we found what was wanted
                }
                
            }
        }
        return requestedRoute;
    }
    
    
    
    /*
     *addRoute will add a route to the routes 
     *list
     *@param destination
     *@param minForwardCount
     *@previousHops
     *@requestStatus
     *@sequeceNumber
     *@source
     */
    public void addRoute(int destination, int minForwardCount,
            LinkedList <Integer> previousHops, boolean requestStatus, int sequenceNumber,
            int source){
        if(routes == null){
            routes = new LinkedList();
        }
	            
            routes.add(new RouteRequested(source,destination,sequenceNumber,
            minForwardCount,previousHops,requestStatus));
        
    }

    
    
    /*
     *getRoutes accessor functions
     */
    public LinkedList<RouteRequested> getRoutes() {
        return routes;
    }

    public void setRoutes(LinkedList<RouteRequested> routes) {
        this.routes = routes;
    }
    
}
