package Sim.Routing.DSR;

import Sim.ProtocolFrame;

/**
 * AckHeader defines a packet that contains only an acknowledgement for a DSR packet. This is used
 * for explicit acknowledgement and is required only for the last hop in a route.
 *
 * @author Jason Winnebeck
 */
public class AckHeader implements ProtocolFrame {
	private final PacketId id;

	/**
	 * Constructs a new ack for the given non-null Packet ID.
	 */
	public AckHeader( PacketId id ) {
		this.id = id;
	}

	/**
	 * Returns the unique PacketId for the packet to be acknowledged.
	 *
	 * @return non-null PacketId
	 */
	public PacketId getId() {
		return id;
	}

	/**
	 * Returns the "serialized" size of this portion of the packet.
	 */
	public int getSize() {
		return id.getSize();
	}
}
