package Sim.Routing.DSR;

import Sim.ProtocolFrame;
import Sim.Packet;
import org.gillius.Util.CheckUtils;

/**
 * ProtocolFrameQueueItem is a QueueItem for a DSR-internal routable packet that does not hold any
 * user data.
*
* @author Jason Winnebeck
*/
class ProtocolFrameQueueItem implements QueueItem {
	private final int dest;
	private final ProtocolFrame pf;

	public ProtocolFrameQueueItem( int dest, ProtocolFrame pf ) {
		CheckUtils.checkNull( pf, "pf" );
		if ( dest < 0 )
		  throw new IllegalArgumentException( "Parameter dest to ProtocolFrameQueueItem.ProtocolFrameQueueItem must be positive or zero, but " + dest + " was given." );

		this.dest = dest;
		this.pf = pf;
	}

	public int getDest() {
		return dest;
	}

	public Packet getPacket() {
		return null;
	}

	public ProtocolFrame getExtraData() {
		return pf;
	}

	public String toString() {
		return "Queued " + pf + " to " + dest;
	}
}
