package Sim.Routing.GPSR;

import Sim.SimulationEvent;
import Sim.Simulation;
import Sim.Node;
import Sim.Routing.GPSR.Header.Broadcast;
import Sim.HardwareFrame;

/**
 * This is the broadcast event class that is used for GPSR nodes
 * that will tell the other neighboring nodes where this node is 
 * located.
 * 
 * @author Benjamin Willis
 */

public class BroadcastEvent implements SimulationEvent {
	/**
	 * node which broadcasts
	 */
	private Node myNode;
	
	/**
	 * Construct a new broadcast event for a given node.
	 * @param myNode node to broadcast
	 */
	public BroadcastEvent(Node myNode){
		this.myNode = myNode;
	}
	
	/**
	 * Method for the event controller to execute.  Sends out a beacon to
	 * all nearby nodes to tell them myNodes location.
	 * 
	 * @param sim simulation that is running
	 * @param current time during the simulation
	 */
	public void perform(Simulation sim, double time) { 
		//send
		PacketId pid = new PacketId();
		HardwareFrame frame = new HardwareFrame(
				myNode.getAddress(), 
				HardwareFrame.BROADCAST, 
				new Broadcast(pid,
						myNode.getAddress(),
						myNode.getLocation()),
				null );

		myNode.sendFrame( frame );
		sim.scheduleEvent( time + 1, this );  //schedule next packet
	} 
	
}