package Sim.Routing.GPSR;

import Sim.Location;

/**
 * Location in the GPSR algorithms table that is expiring.  Once
 * a location expires, that node is not considered nearby anymore.
 * 
 * @author Benjamin Willis
 */
public class ExpiringLocation{
	private Location loc;
	private double timeStored;
	
	/**
	 * Construct a new location and the time it was seen.
	 * 
	 * @param timeStored current time
	 * @param loc location
	 */
	ExpiringLocation(double timeStored, Location loc){
		this.loc = loc;
		this.timeStored = timeStored;
	}
	/**
	 * get the time of this locations storage.
	 * 
	 * @return time stored
	 */
	public double getTimeStored() {
		return timeStored;
	}
	/**
	 * get location of this known neighbor.
	 * @return
	 */
	public Location getLoc() {
		return loc;
	}
	/**
	 *  Change this objects time and location.
	 */
	public void setLoc(Location newLocation,Double time){
		this.timeStored = time;
		this.loc = newLocation;
	}
}