package Sim.Routing.GPSR.Header;

import Sim.Location;
import Sim.Routing.GPSR.PacketId;
/**
 * This is a broadcast header that tells other nodes who this is and
 * what their location is.
 * @author Benjamin Willis
 *
 */
public class Broadcast implements Base {
	public PacketId id;
	private Location fromLocation;
	private int fromAddress;
	
	/**
	 * The constructor to create a new frame for broadcasting to other nodes my current location
	 * and address.
	 * 
	 * @param fromAddress the address of node this was sent from, purely for updating neighbors.
	 * @param fromLocation the location of the sender node, purely for updating neighbors.
	 */
	public Broadcast(PacketId pid, int fromAddress, Location fromLocation ) {
		this.id = pid;
		this.fromLocation = fromLocation;
		this.fromAddress = fromAddress;
	}
	
	/**
	 * Returns the "serialized" size of this portion of the packet.
	 */
	public int getSize() {
		return 1 + 4; //1 for hops, 4 for unique ID (we don't need to count origSource.
	}
	
	/**
	 * The address of the node that sent this header.
	 * @return int of address of sender.
	 */
	public int getFromAddress() {
		return fromAddress;
	}
	
	/**
	 * The location of the node that sent this header.
	 * @return location object of coordinates of originator node.
	 */
	public Location getFromLocation() {
		return fromLocation;
	}

}