package Sim;

/**
 * This location manager gives the position of a node
 * which is moving from a given source to destination
 * point.
 * 
 * @author Benjamin Willis
 */
public class SetLocationManager implements LocationManager{
	private Location end;
	private Location start;
	private Double distance;
	private double startTime;
	private double endTime;
	
	/**
	 * Set location manager allows a node to move from one location
	 * to another location and stop when it gets there
	 * @param speed the speed of the node
	 * @param end end location of the node
	 * @param start begin location of the node
	 */
	public SetLocationManager(Double speed, Location end, Location start) {
		this.end = end;
		this.start = start;
		this.startTime = 0;
		this.distance = start.getDistance(end);
		this.endTime =  this.distance/speed + startTime; 
	}
	
	/**
	 * Get location method based on the path between
	 * the start and end locations and the current time
	 * given.
	 * @param t time to get the location
	 * @return new location object
	 */
	public Location getLocation( double t ) {
		Location newLocation = end;
		if(t < this.endTime)
			{
			double weight = Math.abs((t - this.startTime) / ( this.endTime - this.startTime));
			newLocation = start.mult( 1 - weight ).add( end.mult( weight ) );
			}
		//System.out.println(newLocation.toString());
		return newLocation;
	}
}
