package Sim;

import edu.rit.numeric.prob.DoublePrng;

import java.util.Random;

/**
 * @author Travis
 * @author Jason Winnebeck
 */
public class TrafficGenerator implements SimulationEvent {

	private final World          world;
	private final Random         nodeRng;             //RNG used only to select random nodes
	private       Node           source;              //-1 will mean random destination
	private       Node           destination;         //-1 will mean random destination
	private       int            packetSize  = -1;    //-1 means random size
	private       DoublePrng     periodRand  = null;  //random for period
	private       Boolean        srcToDest   = false;
	private final PacketListener pl;

	/**
	 * Creates a new instance of TrafficGenerator. This instance will be good for sending packets from
	 * one source one destination at a specified interval
	 */
	public TrafficGenerator( Simulation sim, Node source, Node destination, int packetSize,
	                         double startTime, DoublePrng periodRand, PacketListener pl ) {
		this.world = sim.getWorld();
		nodeRng = new Random( sim.getRng().nextLong() );
		this.source = source;
		this.destination = destination;
		this.packetSize = packetSize;
		this.periodRand = periodRand;
		srcToDest = true;
		this.pl = pl;
		sim.scheduleEvent( startTime, this );  //schedule first event
	}

	public TrafficGenerator( Simulation sim, DoublePrng periodRand, int packetSize, double startTime,
	                         PacketListener pl ) {
		this.world = sim.getWorld();
		nodeRng = new Random( sim.getRng().nextLong() );
		this.periodRand = periodRand;
		this.packetSize = packetSize;
		this.pl = pl;

		sim.scheduleEvent( startTime, this );  //make first event
	}

	/**
	 * Will perform a traffic generating event for the simulation.
	 */
	public void perform( Simulation sim, double time ) {
		if ( !srcToDest ) {
			source = world.getRandomNode( nodeRng );
			do {
				destination = world.getRandomNode( nodeRng );
			} while ( source == destination );
		}
		Packet toSend = new Packet( source.getAddress(),
		                            destination.getAddress(),
		                            packetSize,
		                            pl );
		source.sendNewPacket( destination, toSend );          //send a packet
		double t = time + periodRand.next();

		sim.scheduleEvent( t, this );  //schedule next packet
	}
}
