package Sim;

/**
 * Generates a uniform square field of nodes within the center of
 * the area.  Nodes are seperated by half the transmission range.
 * Not that smart of a generator, does not check if the uniform
 * distribution is within the area, just attempts to center the
 * nodes.  Keep the width/height area larger than sqrt(nodes)*tx.
 * 
 * @author Benjamin Willis
 */
public class UniformLocationGenerator implements LocationGenerator{

	//private Integer totalNodes;
	private Double x = 0.0;
	private Double y = 0.0;
	private Double startX = 0.0;
	private Double side;
	private Double widthHeight;
	private Double transmissionRange;
	
	/**
	 * Generates a new UniformLocation structure.  Attempts
	 * to shape the field in a square pattern of nodes in the
	 * middle of the field.
	 * 
	 * @param totalNodes  total number of nodes in the simulation
	 * @param transmissionRange  the range of each node
	 * @param area  the area of the field
	 */
	public UniformLocationGenerator(Integer totalNodes, 
									Double transmissionRange,
									Integer area) {
		//this.totalNodes = totalNodes;
		this.side = (Double) (Math.ceil(Math.sqrt(totalNodes))-1)*Math.sqrt(transmissionRange)-.1;
		this.widthHeight = Math.sqrt(area);
		x = widthHeight/2.0-(side/2.0);
		this.startX = x;
		y = x;
		this.transmissionRange = transmissionRange;
	}

	/**
	 * Gets the next node location in the field thus far.
	 * 
	 * @returns new location object in the uniform field.
	 */
	public Location getNext(){
		//current x,y is already stored 
		Location a = new Location(x,y);
		//calculate the x,y for the next node
		Double newX = x + Math.sqrt(transmissionRange)-.1;
		if(newX >= (startX + side) ){
			//going to next row
			x =	startX;
			y = y + Math.sqrt(transmissionRange)-.1;
		}else{
			x =	newX;
		}
		return a; 
	}
}
