package Sim;

import junit.framework.*;
import Sim.Routing.AbstractRoutingAlgorithm;

import java.util.Random;

/**
 * LoopbackTest
 *
 * @author Jason Winnebeck
 */
@SuppressWarnings( { "FieldCanBeLocal" } )
public class LoopbackTest extends TestCase {

	private Simulation sim;
	private Checker checker;
	private World world;
	private Node node;

	private static final Location location = new Location( 0.0, 0.0 );

	@Override
	protected void setUp() throws Exception {
		super.setUp();

		SimulationParameters params = new SimulationParameters( 0, 0, 1.0, 1.0, 100.0, 200 );

		sim = new Simulation( params );
		world = sim.getWorld();

		LocationManager lm = new FixedLocationManager( location );
		checker = new Checker();
		node = new Node( 0, sim, lm, checker, 10.0, new NodeMonitor() );
		world.addNode( node );
	}

	public void testWorldGetRandomNode() throws Exception {
		Random rng = new Random( 35636 );
		for ( int i = 0; i < 10; ++i ) {
			assertEquals( node, world.getRandomNode( rng ) );
		}
	}

	public void testLoopback() throws Exception {
		Packet p = new Packet( 0, 1, 100 );
		HardwareFrame frame = new HardwareFrame( 0, 1, new EmptyProtocolFrame(), p );
		checker.expected = frame;

		world.sendFrame( frame, location, 5.0 );
		checker.assertChecked();
	}

	private static class Checker extends AbstractRoutingAlgorithm {
		public HardwareFrame expected;
		public boolean checked = false;

		@Override
		public void loopback( HardwareFrame in ) {
			assertEquals( expected, in );
			checked = true;
		}

		public void assertChecked() {
			assertTrue( checked );
			checked = false;
		}
	}
}
